@@ -21,7 +21,7 @@
 import zmq
 
 
-def main(addr, who):
+def main(addr: str, who: str):
 
     ctx = zmq.Context()
     socket = ctx.socket(zmq.PUB)
