@@ -1,5 +1,7 @@
-from __future__ import print_function
+from typing import Optional
+
 from gevent import spawn, spawn_later
+
 import zmq.green as zmq
 
 # server
@@ -10,17 +12,20 @@ sock.bind('ipc:///tmp/zmqtest')
 
 spawn(sock.send_pyobj, ('this', 'is', 'a', 'python', 'tuple'))
 spawn_later(1, sock.send_pyobj, {'hi': 1234})
-spawn_later(2, sock.send_pyobj, ({'this': ['is a more complicated object', ':)']}, 42, 42, 42))
+spawn_later(
+    2, sock.send_pyobj, ({'this': ['is a more complicated object', ':)']}, 42, 42, 42)
+)
 spawn_later(3, sock.send_pyobj, 'foobar')
 spawn_later(4, sock.send_pyobj, 'quit')
 
 
 # client
-ctx = zmq.Context() # create a new context to kick the wheels
+ctx = zmq.Context()  # create a new context to kick the wheels
 sock = ctx.socket(zmq.PULL)
 sock.connect('ipc:///tmp/zmqtest')
 
-def get_objs(sock):
+
+def get_objs(sock: zmq.Socket):
     while True:
         o = sock.recv_pyobj()
         print('received python object:', o)
@@ -28,11 +33,12 @@ def get_objs(sock):
             print('exiting.')
             break
 
-def print_every(s, t=None):
+
+def print_every(s: str, t: Optional[float] = None):
     print(s)
     if t:
         spawn_later(t, print_every, s, t)
 
+
 print_every('printing every half second', 0.5)
 spawn(get_objs, sock).join()
-
