@@ -51,7 +51,7 @@ class SerializingSocket(zmq.Socket):
         """recv a numpy array"""
         md = cast(Dict[str, Any], self.recv_json(flags=flags))
         msg = self.recv(flags=flags, copy=copy, track=track)
-        A = numpy.frombuffer(msg, dtype=md['dtype'])
+        A = numpy.frombuffer(msg, dtype=md['dtype'])  # type: ignore
         return A.reshape(md['shape'])
 
 
