@@ -3,16 +3,20 @@
 # Copyright (C) PyZMQ Developers
 # Distributed under the terms of the Modified BSD License.
 
-from typing import Tuple, Union
+import re
+from typing import Match, Tuple, Union, cast
 
 from zmq.backend import zmq_version_info
 
+__version__: str = "24.0.1"
+_version_pat = re.compile(r"(\d+)\.(\d+)\.(\d+)(.*)")
+_match = cast(Match, _version_pat.match(__version__))
+_version_groups = _match.groups()
 
-VERSION_MAJOR = 22
-VERSION_MINOR = 3
-VERSION_PATCH = 0
-VERSION_EXTRA = ""
-__version__ = '%i.%i.%i' % (VERSION_MAJOR, VERSION_MINOR, VERSION_PATCH)
+VERSION_MAJOR = int(_version_groups[0])
+VERSION_MINOR = int(_version_groups[1])
+VERSION_PATCH = int(_version_groups[2])
+VERSION_EXTRA = _version_groups[3].lstrip(".")
 
 version_info: Union[Tuple[int, int, int], Tuple[int, int, int, float]] = (
     VERSION_MAJOR,
@@ -21,7 +25,6 @@ version_info: Union[Tuple[int, int, int], Tuple[int, int, int, float]] = (
 )
 
 if VERSION_EXTRA:
-    __version__ = "%s.%s" % (__version__, VERSION_EXTRA)
     version_info = (
         VERSION_MAJOR,
         VERSION_MINOR,
@@ -29,13 +32,13 @@ if VERSION_EXTRA:
         float('inf'),
     )
 
-__revision__ = ''
+__revision__: str = ''
 
 
 def pyzmq_version() -> str:
     """return the version of pyzmq as a string"""
     if __revision__:
-        return '@'.join([__version__, __revision__[:6]])
+        return '+'.join([__version__, __revision__[:6]])
     else:
         return __version__
 
